
import { Component, OnInit, OnDestroy, ViewChild, ElementRef, ChangeDetectorRef } from '@angular/core';


import { ProbeService, requesterID } from '../../../commons/service/probe.service';

import { Zone, Host, Port, Service, DiscoverHost } from '@overflow/model/discovery';
import { RPCSubscriber } from '@overflow/commons/ui/decorator/RPCSubscriber';
import { DiscoveryConfigService } from '../../../commons/service/discovery-config.service';
import { Observable } from 'rxjs';

const vis = require('vis');

@Component({
  selector: 'app-pages-home',
  templateUrl: './home-page.component.html',
  styleUrls: ['./home-page.component.scss'],
})
export class HomePageComponent implements OnInit, OnDestroy {

  private discoveryTargetRef: ElementRef;

  zone: Zone;
  discoverHost: DiscoverHost;

  @ViewChild('discoveryTarget') set discoveryTarget(content: ElementRef) {
    this.discoveryTargetRef = content;
  }

  blockedPanel = false;
  displaySidebar = false;

  blockedDocument = false;

  showIntro = true;
  selectedNode = null;

  private visNetwork: any;
  private nodeSet: any;
  private edgeSet: any;

  constructor(
    private changeDetector: ChangeDetectorRef,
    private probeService: ProbeService,
    private discoveryConfigService: DiscoveryConfigService,
  ) {
  }

  ngOnInit() {
    // this.probeService.subscribeNotification(this);
    // this.startDiscovery("");
    this.discoveryConfigService.zone.subscribe(res => {
      this.zone = res as Zone;
    });
    this.discoveryConfigService.discoverHost.subscribe(res => {
      this.discoverHost = res as DiscoverHost;
    });
    this.discoveryConfigService.started.subscribe(res => {
      if (res) {
        this.startDiscovery();
      }
    });
  }

  ngOnDestroy(): void {
    this.probeService.unsubscribeNotification(this);
  }

  blockDocument() {
    this.blockedDocument = true;
    setTimeout(() => {
      this.blockedDocument = false;
    }, 3000);
  }

  startDiscovery() {
    console.log('########################################################');
    console.log(this.zone);
    console.log(this.discoverHost);
    console.log('########################################################');
    this.showIntro = false;
    this.changeDetector.detectChanges();

    this.nodeSet = new vis.DataSet([
      {
        id: '192.168.1.0/24', label: 'Zone', group: 'zone',
        shape: "image",
        image: "../../assets/Windows_logo.png",
      },
    ]);

    this.edgeSet = new vis.DataSet([]);

    const options = {
      width: '100%',
      height: '100%',
      groups: {
        zone: { color: { background: 'red' }, borderWidth: 5 },
        host: { color: { background: 'blue' }, borderWidth: 3 },
        service: { color: { background: 'white' }, borderWidth: 1 },
      }
    };

    this.visNetwork = new vis.Network(this.discoveryTargetRef.nativeElement, { nodes: this.nodeSet, edges: this.edgeSet }, options);
    this.visNetwork.on('click', (params) => {
      const ids = params.nodes;
      const clickedNode = this.nodeSet.get(ids);
      if (clickedNode.length > 0) {
        this.displaySidebar = true;
        this.selectedNode = clickedNode[0];
      } else {
        this.displaySidebar = false;
        this.selectedNode = null;
      }
    });

    // this.probeService.send(
    //   'DiscoveryService.DiscoverHost',
    //   requesterID,
    //   this.discoveryConfigService.getZone(),
    //   this.discoveryConfigService.getDiscoverHost()
    // );
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.Queueing' })
  public DiscoveryQueueing(queueingDate: Date) {
    console.log('DiscoveryQueueing', queueingDate);
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.QueueingFailed' })
  public DiscoveryQueueingFailed(queueingFailedDate: Date) {
    console.log('DiscoveryQueueingFailed', queueingFailedDate);
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.QueueingTimeout' })
  public DiscoveryQueueingTimeout(queueingTimeoutDate: Date) {
    console.log('DiscoveryQueueingTimeout', queueingTimeoutDate);
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.DiscoveryStart' })
  public DiscoveryStart(startDate: Date) {
    console.log('DiscoveryStart', startDate);
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.DiscoveryStop' })
  public DiscoveryStop(stopDate: Date) {
    console.log('DiscoveryStop', stopDate);
    this.visNetwork.stabilize();
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.DiscoveryError' })
  public DiscoveryError(err: Error) {
    console.log('DiscoveryError', err);
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.DiscoveredHost' })
  public DiscoveredHost(host: Host) {
    console.log('DiscoveredHost', host);
    const hostId = `${host.address}`;

    if (null !== this.nodeSet.get(hostId)) {
      this.nodeSet.update([
        { id: hostId, label: `${host.name}(${host.address})`, group: 'host', host: host },
      ]);
    } else {
      this.nodeSet.add([
        { id: hostId, label: `${host.name}(${host.address})`, group: 'host', host: host },
      ]);
      this.edgeSet.add([
        { from: '192.168.1.0/24', to: hostId },
      ]);
    }

  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.DiscoveredPort' })
  public DiscoveredPort(port: Port) {
    console.log('DiscoveredPort', port);
  }

  /**
   * DiscoverHost
   */
  @RPCSubscriber({ method: 'DiscoveryService.DiscoveredService' })
  public DiscoveredService(service: Service) {
    console.log('DiscoveredService', service);
    const hostId = service.port.host.address;
    const serviceId = `${service.port.host.address}-${service.port.portNumber}-${service.key}`;
    this.nodeSet.add([
      { id: `${serviceId}`, label: `${service.key}`, group: 'service', service: service }
    ]);
    this.edgeSet.add([
      { from: `${hostId}`, label: `${service.port.portNumber}`, to: `${serviceId}` },
    ]);

  }
}
